# -*- coding: utf-8 -*-
import json
import sentiment_analysis_functions as sfun
from scipy.sparse import csr_matrix, hstack

# installation
# import nltk
# nltk.download('punkt')
# nltk.download('wordnet')
# nltk.download('stopwords')
# nltk.download('averaged_perceptron_tagger')
# nltk.download('universal_tagset')
# nltk.download('sentiwordnet')


# from sklearn.naive_bayes import MultinomialNB
# from sklearn.linear_model import LogisticRegression

train_pos_reviews_fn = "./data/train-positive-t1.txt"
train_neg_reviews_fn = "./data/train-negative-t1.txt"
test_pos_reviews_fn = "./data/test-pos-t1.txt"
test_neg_reviews_fn = "./data/test-neg-t1.txt"


# Aucune contrainte pour cette tâche. Vous pouvez structurez votre code comme bon vous semble.
# Expliquez dans votre rapport comment exécuter le code et définir les configurations.


def load_reviews(filename):
    with open(filename, 'r') as fp:
        reviews_list = json.load(fp)
    return reviews_list


if __name__ == '__main__':
    train_positive_reviews = load_reviews(train_pos_reviews_fn)
    train_negative_reviews = load_reviews(train_neg_reviews_fn)
    test_positive_reviews = load_reviews(test_pos_reviews_fn)
    test_negative_reviews = load_reviews(test_neg_reviews_fn)
    print("Nb of training reviews - positive:", len(train_positive_reviews), "negative:", len(train_negative_reviews))
    print("Nb of test reviews - positive:", len(test_positive_reviews), "negative:", len(test_negative_reviews))
    
    # Train and test datasets
    
    train_dataset = train_positive_reviews+train_negative_reviews
    train_dataset_response = [1]*len(train_positive_reviews)+[0]*len(train_negative_reviews)
    
    test_dataset = test_positive_reviews+test_negative_reviews
    test_dataset_response = [1]*len(test_positive_reviews)+[0]*len(test_negative_reviews)

    # Tokenisation
    
    # Tokenize train
    train_tokens = sfun.tokenize(train_dataset)
    norm_train_tokens = []
    norm_train_tokens.append(sfun.norm_stemming(train_tokens))
    norm_train_tokens.append(sfun.norm_lemmatize(train_tokens))
    
    # Tokenize test
    test_tokens = sfun.tokenize(test_dataset)
    norm_test_tokens = []
    norm_test_tokens.append(sfun.norm_stemming(test_tokens))
    norm_test_tokens.append(sfun.norm_lemmatize(test_tokens))   
    
    
    # Normalize and select tokens
    norm_select_train_tokens = []
    norm_select_test_tokens = []
    norm_select_train_tokens_split = []
    norm_select_test_tokens_split = []
    for norm_method in range(0,2):
        # train tokens
        select_train_tokens = []
        select_train_tokens_split = []
        nn1 = sfun.select_freq(norm_train_tokens[norm_method])
        nn2 = sfun.select_rem_stopwords(norm_train_tokens[norm_method])
        nn3 = sfun.select_open_class(norm_train_tokens[norm_method])
        select_train_tokens_split.append(nn1)
        select_train_tokens_split.append(nn2)
        select_train_tokens_split.append(nn3)
        select_train_tokens.append(sfun.corpus_documents(nn1))
        select_train_tokens.append(sfun.corpus_documents(nn2))
        select_train_tokens.append(sfun.corpus_documents(nn3))       
        norm_select_train_tokens_split.append(select_train_tokens_split)
        norm_select_train_tokens.append(select_train_tokens)
        # test tokens
        select_test_tokens = []
        select_test_tokens_split = []
        nn1 = sfun.select_freq(norm_test_tokens[norm_method])
        nn2 = sfun.select_rem_stopwords(norm_test_tokens[norm_method])
        nn3 = sfun.select_open_class(norm_test_tokens[norm_method])
        select_test_tokens_split.append(nn1)
        select_test_tokens_split.append(nn2)
        select_test_tokens_split.append(nn3)
        select_test_tokens.append(sfun.corpus_documents(nn1))
        select_test_tokens.append(sfun.corpus_documents(nn2))
        select_test_tokens.append(sfun.corpus_documents(nn3))       
        norm_select_test_tokens_split.append(select_test_tokens_split)
        norm_select_test_tokens.append(select_test_tokens)
    
    # Création des objets Vectorizer (Train seulement)
    norm_select_vectorizers = []
    for norm_method in range(0,2):
        select_vectorizers = []
        for select_method in range(0,3):
            vectorizers = sfun.get_vectorizers(norm_select_train_tokens[norm_method][select_method])
            select_vectorizers.append(vectorizers)
        norm_select_vectorizers.append(select_vectorizers)
    
    # Transformation des jetons en vecteurs
    v_norm_select_vectors_train = []
    v_norm_select_vectors_test = []
    for norm_method in range(0,2):
        v_select_vectors_train = []
        v_select_vectors_test = []        
        for select_method in range(0,3):
            v_vectors_train = []
            v_vectors_test = []
            for vector_method in range(0,3):
                v_vector_train = norm_select_vectorizers[norm_method][select_method][vector_method].transform(norm_select_train_tokens[norm_method][select_method])
                v_vector_test = norm_select_vectorizers[norm_method][select_method][vector_method].transform(norm_select_test_tokens[norm_method][select_method])
                v_vectors_train.append(v_vector_train)
                v_vectors_test.append(v_vector_test)
            v_select_vectors_train.append(v_vectors_train)
            v_select_vectors_test.append(v_vectors_test)
        v_norm_select_vectors_train.append(v_select_vectors_train)
        v_norm_select_vectors_test.append(v_select_vectors_test)
        
    # Ajout des attributs
    v_norm_select_polarity_count_train = []
    v_norm_select_polarity_count_test = []
    for norm_method in range(0,2):
        v_select_polarity_count_train = []
        v_select_polarity_count_test = []
        for select_method in range(0,3):
            v_polarity_count_train = sfun.attribute_polarity_count(norm_select_train_tokens_split[norm_method][select_method])
            v_polarity_count_test = sfun.attribute_polarity_count(norm_select_test_tokens_split[norm_method][select_method])
            v_select_polarity_count_train.append(csr_matrix(v_polarity_count_train))
            v_select_polarity_count_test.append(csr_matrix(v_polarity_count_test))
        v_norm_select_polarity_count_train.append(v_select_polarity_count_train)
        v_norm_select_polarity_count_test.append(v_select_polarity_count_test)
    
    # Création des matrices finales
    v_final_train = []
    v_final_test = []
    for norm_method in range(0,2):
        v_select_final_train = []
        v_select_final_test = []
        for select_method in range(0,3):
            v_vector_final_train = []
            v_vector_final_test = []            
            for vector_method in range(0,3):
                v_vector_final_train.append(hstack([v_norm_select_vectors_train[norm_method][select_method][vector_method],
                                                v_norm_select_polarity_count_train[norm_method][select_method]]))
                v_vector_final_test.append(hstack([v_norm_select_vectors_test[norm_method][select_method][vector_method],
                                               v_norm_select_polarity_count_test[norm_method][select_method]]))
            v_select_final_train.append(v_vector_final_train)
            v_select_final_test.append(v_vector_final_test)
        v_final_train.append(v_select_final_train)
        v_final_test.append(v_select_final_test)
            
    # Scoring des modèles
